# BIP66

> A simple C++ implementation of Bitcoin BIP66 for the Ark Ecosystem.

ANS.1 DER Encoder/Decoder based on [Bitcoin BIP66](https://github.com/bitcoin/bips/blob/master/bip-0066.mediawiki).

[![Build Status](https://badgen.now.sh/circleci/github/sleepdefic1t/BIP66)](https://circleci.com/gh/sleepdefic1t/BIP66)
[![Latest Version](https://badgen.now.sh/github/release/sleepdefic1t/BIP66)](https://github.com/sleepdefic1t/BIP66/releases)
[![License: MIT](https://badgen.now.sh/badge/license/MIT/green)](https://opensource.org/licenses/MIT)

## Supported Platforms

| IoT: | supported: |
| :-------- | :--------: |
| Arduino IDE | [x] |
| PlatformIO | [x] |

| Boards: | supported: |
| :-------- | :--------: |
| ESP8266 | [x] |
| ESP32 | [x] |

| OS Platform: | supported: |
| :-------- | :--------: |
| Linux | [x] |
| macOS | [x] |
| Windows | [x] |

## Installation

### Dependencies

- IoT:
  - **Arduino IDE**: N/A
  - **PlatformIO**: AUnit
- OS:
  - **Linux**: CMake
  - **macOS**: CMake
  - **Windows**: CMake

---

### Arduino IDE

1) Open the `BIP66.ino` sketch in the `examples/BIP66` folder.  
2) Select your board from the Arduino IDE
3) Upload the sketch.

---

### PlatformIO

1) `cd test`
2) `cmake --build .`
3) `pio run -e esp32 -t upload`<br>_or_ `pio run -e esp8266 -t upload`

---

### OS Builds

Operating System builds like Linux, macOS, and Windows use CMake to build this BIP66 library.

1) `cmake .`
2) `cmake --build .`
3) `./test/BIP66_tests`

---

## Usage

### Check/Validate an Encoded Signature

From a DER encoded signature:

```cpp
std::vector<uint8_t> derEncodedSignature = {
    0x30, 0x44, 0x02, 0x20, 0x21, 0x70, 0x4f, 0x2a,
    0xdb, 0x2e, 0x4a, 0x10, 0xa3, 0xdd, 0xc1, 0xd7,
    0xd6, 0x45, 0x52, 0xb8, 0x06, 0x1c, 0x05, 0xf6,
    0xd1, 0x2a, 0x16, 0x8c, 0x69, 0x09, 0x1c, 0x75,
    0x58, 0x1d, 0x61, 0x14, 0x02, 0x20, 0x0e, 0xdf,
    0x37, 0x68, 0x9d, 0x27, 0x86, 0xfc, 0x69, 0x0a,
    0xf9, 0xf0, 0xf6, 0xfa, 0x1f, 0x62, 0x9c, 0x95,
    0x69, 0x50, 0x39, 0xf6, 0x48, 0xa6, 0xd4, 0x55,
    0x48, 0x43, 0x02, 0x40, 0x2e, 0x93
};

const bool checked = BIP66::check(derEncodedSignature);
```

---

### Decode a Signature

From a DER encoded signature to raw r-value & s-value:

```cpp
std::vector<uint8_t> derEncodedSignature = {
    0x30, 0x44, 0x02, 0x20, 0x21, 0x70, 0x4f, 0x2a,
    0xdb, 0x2e, 0x4a, 0x10, 0xa3, 0xdd, 0xc1, 0xd7,
    0xd6, 0x45, 0x52, 0xb8, 0x06, 0x1c, 0x05, 0xf6,
    0xd1, 0x2a, 0x16, 0x8c, 0x69, 0x09, 0x1c, 0x75,
    0x58, 0x1d, 0x61, 0x14, 0x02, 0x20, 0x0e, 0xdf,
    0x37, 0x68, 0x9d, 0x27, 0x86, 0xfc, 0x69, 0x0a,
    0xf9, 0xf0, 0xf6, 0xfa, 0x1f, 0x62, 0x9c, 0x95,
    0x69, 0x50, 0x39, 0xf6, 0x48, 0xa6, 0xd4, 0x55,
    0x48, 0x43, 0x02, 0x40, 0x2e, 0x93
};

std::vector<uint8_t> r(32);
std::vector<uint8_t> s(32);

const bool decoded = BIP66::decode(derEncodedSignature, r, s);
```

---

### Encode a Signature

From the raw r-value & s-value:

```cpp
std::vector<uint8_t> r = {
    0x21, 0x70, 0x4f, 0x2a, 0xdb, 0x2e, 0x4a, 0x10,
    0xa3, 0xdd, 0xc1, 0xd7, 0xd6, 0x45, 0x52, 0xb8,
    0x06, 0x1c, 0x05, 0xf6, 0xd1, 0x2a, 0x16, 0x8c,
    0x69, 0x09, 0x1c, 0x75, 0x58, 0x1d, 0x61, 0x14,
};

std::vector<uint8_t> s = {
    0x0e, 0xdf, 0x37, 0x68, 0x9d, 0x27, 0x86, 0xfc,
    0x69, 0x0a, 0xf9, 0xf0, 0xf6, 0xfa, 0x1f, 0x62,
    0x9c, 0x95, 0x69, 0x50, 0x39, 0xf6, 0x48, 0xa6,
    0xd4, 0x55, 0x48, 0x43, 0x02, 0x40, 0x2e, 0x93
};

std::vector<uint8_t> rawSignatureBuffer(64);

const bool encoded = BIP66::encode(r, s, rawSignatureBuffer);
```

From a raw unencoded signature:

```cpp
std::vector<uint8_t> rawUnencodedSignature = {
    0x21, 0x70, 0x4f, 0x2a, 0xdb, 0x2e, 0x4a, 0x10,
    0xa3, 0xdd, 0xc1, 0xd7, 0xd6, 0x45, 0x52, 0xb8,
    0x06, 0x1c, 0x05, 0xf6, 0xd1, 0x2a, 0x16, 0x8c,
    0x69, 0x09, 0x1c, 0x75, 0x58, 0x1d, 0x61, 0x14,
    0x0e, 0xdf, 0x37, 0x68, 0x9d, 0x27, 0x86, 0xfc,
    0x69, 0x0a, 0xf9, 0xf0, 0xf6, 0xfa, 0x1f, 0x62,
    0x9c, 0x95, 0x69, 0x50, 0x39, 0xf6, 0x48, 0xa6,
    0xd4, 0x55, 0x48, 0x43, 0x02, 0x40, 0x2e, 0x93
};

std::vector<uint8_t> encodedSignatureBuffer(72);

const bool encoded = BIP66::encode(&rawUnencodedSignature[0], encodedSignatureBuffer);
```

---

## Security

If you discover a security vulnerability within this package, please send an e-mail to security@ark.io. All security vulnerabilities will be promptly addressed.

## Credits

- [Simon Downey](https://github.com/sleepdeficit)
- [Chris Johnson](https://github.com/ciband)
- [All Contributors](../../../../contributors)

## License

[MIT](LICENSE) © [ArkEcosystem](https://ark.io)
